----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    11:48:25 12/03/2007 
-- Design Name: 
-- Module Name:    automaton - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity automaton is
	
	-- I/O SIGNALI
	port(
		CLK_in	: in STD_LOGIC;
		RST_in	: in STD_LOGIC;
		DATA_in	: in STD_LOGIC_VECTOR(7 downto 0);
		DATA_RDY_in : in STD_LOGIC;
		
		-- izhodi proti LCD-ju
		LCD_E_out	: out	STD_LOGIC;
		LCD_RS_out	: out	STD_LOGIC;
		LCD_RW_out	: out	STD_LOGIC;
		SF_D_inout	: inout	STD_LOGIC_VECTOR(11 downto 8)
	--	LED			: out STD_LOGIC_VECTOR(3 downto 0)
	);


end automaton;

architecture Behavioral of automaton is

-- COMPONENT DESCRIPTION
	COMPONENT lcd_module
	PORT(
		CLK_in : IN std_logic;
		RST_in : IN std_logic;
		DATA_RDY_in : IN std_logic;
		DATA_TO_SEND_in : IN std_logic_vector(9 downto 0);    
		SF_D_inout : INOUT std_logic_vector(11 downto 8);      
		LCD_READY_out : OUT std_logic;
		LCD_E_out : OUT std_logic;
		LCD_RS_out : OUT std_logic;
		LCD_RW_out : OUT std_logic
		);
	END COMPONENT;
	
		
-- INTERNAL SIGNALS
	signal LCD_E	: STD_LOGIC;
	signal LCD_RS	: STD_LOGIC;
	signal LCD_RW	: STD_LOGIC;
	signal SF_D	: STD_LOGIC_VECTOR(11 downto 8);
	signal DATA_RDY	: STD_LOGIC;
	signal LCD_READY	: STD_LOGIC;
	signal DATA_TO_SEND	:	STD_LOGIC_VECTOR(9 downto 0);
	
	-- AUTOMATON STATES
	type states is (idle, send);
	signal currSt, nextSt : states;
	signal nextREADY : STD_LOGIC;
	signal nextDATA : STD_LOGIC_VECTOR(9 downto 0);
	signal LCD_DATA : STD_LOGIC_VECTOR(9 downto 0);

begin

-- IO ASSIGNMENT
		
		LCD_E_out	<= LCD_E;
		LCD_RS_out	<= LCD_RS;
		LCD_RW_out	<= LCD_RW;
		SF_D_inout	<= SF_D;
		LCD_DATA		<= "10" & DATA_in;

-- COMPONENT INSTANTIATION
	LCD: lcd_module PORT MAP(
		CLK_in => CLK_in,
		RST_in => RST_in,
		DATA_RDY_in =>	DATA_RDY,
		LCD_READY_out =>	LCD_READY,
		DATA_TO_SEND_in =>	DATA_TO_SEND,
		LCD_E_out =>	LCD_E,
		LCD_RS_out =>	LCD_RS,
		LCD_RW_out =>	LCD_RW,
		SF_D_inout =>	SF_D 
	);
	
-- AUTOMATON
	process (CLK_in)
	begin
		if (rising_edge(CLK_in)) then
			if (RST_in = '1') then
				currSt	<= idle;
				DATA_RDY <= '0';
				DATA_TO_SEND <= (others => '0');
			else
				currSt	<= nextSt;
				DATA_RDY <= nextREADY;
				DATA_TO_SEND <= nextDATA;
			end if;
		end if;	
	end process;

	transition_func: process (currSt, LCD_READY, DATA_RDY_in)
	begin
		case currSt is
		-- IDLE
		when idle =>
			if (DATA_RDY_in = '1') then
				nextSt	<= send;
			else
				nextSt	<= idle;
			end if;
		-- SEND
		when send =>
			if (LCD_READY = '1') then
				nextSt	<= idle;
			else
				nextSt	<= send;
			end if;
		end case;
	end process;
	
	output_func:process (currSt, LCD_READY, DATA_RDY_in)
	begin
		case currSt is
		-- IDLE
		when idle =>
			if (DATA_RDY_in = '1') then	-- SENDING
				nextREADY	<= '1';
				nextDATA		<= LCD_DATA;
			else
				nextREADY	<= '0';
				nextDATA		<= (others => '0');
			end if;
		-- SEND
		when send =>
			if (LCD_READY = '1') then
				nextREADY	<= '0';
				nextDATA		<= (others => '0');
			else
				nextREADY	<= '1';
				nextDATA		<= LCD_DATA;
			end if;
		end case;		
	end process;
	
	
end Behavioral;

