----------------------------------------------------------------------------------
-- Company: FRI
-- Engineer: Miha Moskon
-- 
-- Create Date:    11:26:51 09/28/2007 
-- Design Name: 
-- Module Name:    lcd_init - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity lcd_init is
	port(
		CLK_in	: in	STD_LOGIC;		-- ura
		RST_in	: in	STD_LOGIC;		-- skupni reset signal
		
		LCD_E_out	: out	STD_LOGIC;		-- lcd enable 
		SF_D_inout	: inout	STD_LOGIC_VECTOR(11 downto 8);	-- podatkovni signali LCD-ja
		
		INIT_IN_PROG_in	: in	STD_LOGIC;	-- odvija se inicializacija
		INIT_DONE_out	: out	STD_LOGIC);		-- inicializacija je koncana

end lcd_init;

architecture Behavioral of lcd_init is
	
	component counter
		generic(
			size : integer := 8);
		port(
			CLK_in	: IN STD_LOGIC;
			WE_in	: IN STD_LOGIC;
			VALUE_in	: IN STD_LOGIC_VECTOR(size-1 downto 0);
			COUNT_UP_out	: OUT STD_LOGIC);		
	end component;
	
	-- stanja init avtomata
	type STATES is (DUMMY, DELAY11, DELAY12, SEND11, SEND12, DELAY21, DELAY22, SEND21, SEND22, DELAY31, DELAY32, SEND31, SEND32, DELAY41, DELAY42, SEND41, SEND42, DELAY51, DELAY52, DONE);	-- definiranje moznih stanj avtomata
	signal currSt, nextSt	:	STATES;		-- trenutno in naslednje stanje
	
	-- signali za stevec
	signal WE_counter, count_up	: STD_LOGIC;
	signal value_to_counter	: STD_LOGIC_VECTOR(19 downto 0);	-- znati mora steti do 750.000 (20 bitov)
	
	-- IO assignment signali
	signal LCD_E	: STD_LOGIC;		-- lcd enable 
	signal SF_D	: STD_LOGIC_VECTOR(11 downto 8);	-- podatkovni signali LCD-ja
	signal INIT_DONE	: STD_LOGIC;		-- inicializacija je koncana
	
begin

-- IO ASSIGNMENT

	LCD_E_out	<= LCD_E;
	SF_D_inout	<= SF_D;
	INIT_DONE_out	<= INIT_DONE;


-- INSTANCIRANJE KOMPONENT
	-- instanca stevca
	counter20bit:counter
		generic map
		( 
			size => 20		-- 20-bitni stevec: znati mora steti do 750.000
		)
		port map
		(
			CLK_in	=> CLK_in,
			WE_in	=> WE_counter,
			VALUE_in	=> value_to_counter,
			COUNT_UP_out	=> count_up		
		);


-- INIT AVTOMAT:	
	
	sync_proc:process (CLK_in)
	begin
		if (rising_edge(CLK_in)) then
			if ( (RST_in = '1') OR (INIT_IN_PROG_in = '0') ) then 
				currSt	<= DUMMY;
			else
				currSt	<= nextSt;
			end if;
		end if;
	end process;
	
	init_proc: process (currSt, INIT_IN_PROG_in, count_up)
	begin
		-- lcd		
		SF_D <= "0000";
		LCD_E <= '0';
		
		-- stevec
		WE_counter <= '0';
		value_to_counter <= x"00000";
		
		-- izhod iz avtomata v lcd_top
		INIT_DONE <= '0';
		
		
		case currSt is
		
			-- dummy state (v njem je avtomat, ko je init_in_prog_out = 0)
			when DUMMY =>
				if (INIT_IN_PROG_in = '1') then
					nextSt <= DELAY11;					
				else
					nextSt <= DUMMY;			
				end if;

			-- delay 1 init
			when DELAY11 =>
				WE_counter	<= '1';
				value_to_counter <= x"B71B0"; -- 750000 urinih period (15ms)

				nextSt	<= DELAY21;				
			
			-- wait for delay 1
			when DELAY12 =>
				if (count_up = '0') then
					nextSt	<= DELAY12;
				else
					nextSt	<= SEND11;
				end if;
			
			-- send 1 init
			when SEND11 =>
				WE_counter	<= '1';
				value_to_counter <= x"0000C";	-- 12 urinih period
				
				nextSt	<= SEND21;
			
			-- send 1
			when SEND12 =>
				SF_D	<=	"0011";
				LCD_E	<= '1';
							
				if (count_up = '0') then
					nextSt	<= SEND12;
				else
					nextSt	<= DELAY21;
				end if;
			
			-- delay 2 init
			when DELAY21 =>
				WE_counter	<= '1';
				value_to_counter	<= x"320C8";	-- 205000 urinih period (4,1ms)
				
				nextSt <= DELAY22;
			
			-- wait for delay 2
			when DELAY22 =>
				if (count_up = '0') then
					nextSt <= DELAY22;
				else
					nextSt <= SEND21;
				end if;
			
			-- send 2 init
			when SEND21 =>
				WE_counter	<= '1';
				value_to_counter <= x"0000C";	-- 12 urinih period 
				
				nextSt	<= SEND22;
			
			-- send 2
			when SEND22 =>
				SF_D	<=	"0011";
				LCD_E	<= '1';
				
				if (count_up = '0') then
					nextSt	<= SEND22;
				else
					nextSt	<= DELAY31;
				end if;
							
			-- delay 3 init
			when DELAY31 =>
				WE_counter	<= '1';
				value_to_counter	<= x"01388"; -- 5000 urinih period (100us)
				
				nextSt	<= DELAY32;
			
			-- wait for delay 3
			when DELAY32 =>
				if (count_up = '0') then
					nextSt	<= DELAY32;
				else
					nextSt	<= SEND31;
				end if;
			
			-- send 3 init
			when SEND31 =>
				WE_counter	<= '1';
				value_to_counter	<= x"0000C";	-- 12 urinih period
				
				nextSt	<= SEND32;
			
			-- send 3
			when SEND32 =>
				SF_D	<=	"0011";
				LCD_E	<= '1';
				
				if (count_up = '0') then
					nextSt	<= SEND32;
				else
					nextSt	<= DELAY41;
				end if;
			
			-- delay 4 init
			when DELAY41 =>
				WE_counter	<= '1';
				value_to_counter	<= x"007D0";	-- 2000 urinih period (40us)
				
				nextSt	<= DELAY42;
				
			-- wait for delay 4
			when DELAY42 =>
				if (count_up = '0') then
					nextSt	<= DELAY42;
				else
					nextSt	<= SEND41;
				end if;
			
			-- send 4 init
			when SEND41 =>
				WE_counter	<= '1';
				value_to_counter	<= x"0000C";	-- 12 urinih period 
				
				nextSt	<= SEND42;
			
			-- send 4
			when SEND42 =>
				SF_D	<=	"0010";
				LCD_E	<= '1';
				
				if (count_up = '0') then
					nextSt	<= SEND42;
				else
					nextSt	<= DELAY51;
				end if;
			
			-- delay 5 init
			when DELAY51 =>
				WE_counter	<= '1';
				value_to_counter	<= x"007D0";	-- 2000 urinih period (40us)
			
				nextSt	<= DELAY52;
			
			-- wait for DELAY 5 - then transit to DONE
			when DELAY52 =>
				if (count_up = '0') then
					nextSt	<= DELAY52;
				else
					nextSt	<= DONE;
				end if;	
			
			-- init done - inicializacija opravljena
			when DONE	=>
				INIT_DONE <= '1';
				
				nextSt	<= DUMMY;

		end case;

	end process;

end Behavioral;

